/**
 * Copyright 2010-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.protocol.osgi.impl;

import java.util.Dictionary;
import java.util.Hashtable;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;
import org.ow2.util.protocol.api.IProtocol;
import org.ow2.util.protocol.api.IProtocolConfigRegistry;
import org.ow2.util.protocol.api.IShellFormatter;
import org.ow2.util.protocol.impl.ProcessProtocol;
import org.ow2.util.protocol.impl.ProtocolConfigRegistryImpl;
import org.ow2.util.protocol.impl.shell.MsDosShellFormatter;
import org.ow2.util.protocol.impl.shell.ShShellFormatter;
import org.ow2.util.protocol.osgi.api.Constants;

/**
 *
 * @author Loris Bouzonnet
 */
public class Activator implements BundleActivator {

    private ServiceRegistration<IProtocol> processReg;

    private ServiceRegistration<IShellFormatter> shReg;

    private ServiceRegistration<IShellFormatter> msDosReg;

    private ServiceRegistration<IProtocolConfigRegistry> registryReg;

    public void start(final BundleContext context) throws Exception {

        final Dictionary<String, String> processProps = new Hashtable<String, String>();
        processProps.put(Constants.PROTOCOL_NAME, Constants.PROCESS_PROTOCOL);
        processReg = context.registerService(IProtocol.class, new ProcessProtocol(), processProps);

        final ShShellFormatter shFormatter = new ShShellFormatter();
        final Dictionary<String, String> shProps = new Hashtable<String, String>();
        shProps.put(Constants.SHELL_TYPE, "sh");
        shReg = context.registerService(IShellFormatter.class, shFormatter, shProps);

        final MsDosShellFormatter msDosFormatter = new MsDosShellFormatter();
        final Dictionary<String, String> msDosProps = new Hashtable<String, String>();
        msDosProps.put(Constants.SHELL_TYPE, "msdos");
        msDosReg = context.registerService(IShellFormatter.class, msDosFormatter, msDosProps);

        final ProtocolConfigRegistryImpl configRegistry = new ProtocolConfigRegistryImpl();
        registryReg = context.registerService(IProtocolConfigRegistry.class, configRegistry, null);

    }

    public void stop(final BundleContext context) throws Exception {

        if (processReg != null) {
            processReg.unregister();
        }
        if (shReg != null) {
            shReg.unregister();
        }
        if (msDosReg != null) {
            msDosReg.unregister();
        }
        if (registryReg != null) {
            registryReg.unregister();
        }
    }

}
